import { DataSettings, ViewSettings } from 'ptnl-constructor-sdk';
import {
    checkbox,
    colorPicker,
    CreateViewSettings,
    input,
    select,
    textArea,
    ViewSettingsItem,
    ViewSettingsValidation,
} from 'ptnl-constructor-sdk/config';
import { digitCapacityDefault, digitCapacityMax } from './constants';
import { EChartType, EValueType, EViewOption } from './enum';
const fontSize = [
    ['12', '12', '12'],
    ['13', '13', '13'],
    ['14', '14', '14'],
    ['15', '15', '15'],
    ['16', '16', '16'],
    ['17', '17', '17'],
    ['18', '18', '18'],
    ['19', '19', '19'],
    ['20', '20', '20'],
    ['21', '21', '21'],
    ['22', '22', '22'],
    ['23', '23', '23'],
    ['24', '24', '24'],
    ['25', '25', '25'],
    ['26', '26', '26'],
    ['27', '27', '27'],
    ['28', '28', '28'],
    ['29', '29', '29'],
    ['30', '30', '30'],
];
const typeChart = [
    [EChartType.HorizonatalType, 'Горизонтальный', 'Horizontal'],
    [EChartType.VerticalType, 'Вертикальный', 'Vertical'],
];

const typeValue = [
    [EValueType.Change, 'Изменение (A-B)', 'Change (A-B)'],
    [
        EValueType.ChangeInPercents,
        'Изменение в процентах ((A-B)/B*100)',
        'Percentage change ((A-B)/B*100)',
    ],
    [
        EValueType.PlanFact,
        'Факт/План ((A/B) * 100)',
        'Fact / Plan ((A/B) * 100)',
    ],
    [
        EValueType.ResultSegment,
        'Результат деления (A/B)',
        'Result of division (A/B)',
    ],
    [
        EValueType.ChangeDole,
        'Изменение долей ((А/B * 100) - (A1/B1 * 100))',
        'Change in shares ((А/B * 100) - (A1/B1 * 100))',
    ],
    [
        EValueType.CalculatedValue,
        'Изменение в процентах (расчетный) (((А/B) - (A1/B1)) / (A1/B1) * 100)',
        'Percentage change (calculated) (((А/B) - (A1/B1)) / (A1/B1) * 100)',
    ],
    [
        EValueType.DifferenceSegment,
        'Разница результата делений ((A/B) - (A1/B1))',
        'Division result difference ((A/B) - (A1/B1))',
    ],
];

export const createViewSettings: CreateViewSettings<DataSettings> = ({
    dataSettings,
    viewSettings,
}: {
    dataSettings: DataSettings;
    viewSettings: ViewSettings;
}) => {
    return [
        select({
            key: EViewOption.ChartType,
            label: {
                ru: 'Вариант отображения',
                en: 'Display option',
            },
            defaultValue: typeChart[1][0],
            options: typeChart.map((item) => {
                const [value, ru, en] = item;
                return {
                    label: {
                        ru,
                        en,
                    },
                    value,
                };
            }),
        }),
        select({
            key: EViewOption.TypeValue,
            label: {
                ru: 'Вариант расчета показателя',
                en: 'Indicator calculation option',
            },
            defaultValue: typeValue[1][0],
            options: typeValue.map((item) => {
                const [value, ru, en] = item;
                return {
                    label: {
                        ru,
                        en,
                    },
                    value,
                };
            }),
        }),
        checkbox({
            key: EViewOption.ShowFactName,
            label: {
                ru: 'Отображать наименование в теле виджета',
                en: 'Display title in the body of the widget',
            },
            defaultValue: true,
        }),
        // checkbox({
        //     key: EViewOption.PercentageDiff,
        //     label: {
        //         ru: 'Разница в проценте',
        //         en: 'Percentage difference',
        //     },
        //     defaultValue: true,
        // }),
        checkbox({
            key: EViewOption.EnableDigitCapacity,
            label: {
                ru: 'Включить разрядность значений',
                en: 'Enable digit capacity',
            },
            defaultValue: true,
        }),
        input({
            key: EViewOption.DigitCapacity,
            label: {
                ru: 'Разрядность значений',
                en: 'Digit capacity',
            },
            defaultValue: `${digitCapacityDefault}`,
        }),
        input({
            key: EViewOption.Name,
            label: {
                ru: 'Наименование виджете в теле',
                en: 'Name of the widget in the body',
            },
        }),
        input({
            key: EViewOption.Description,
            label: {
                ru: 'Описание виджета в теле',
                en: 'Description of the widget in the body',
            },
        }),

        select({
            key: EViewOption.FontSizeIndex,
            label: {
                ru: 'Размер шрифта показателя с шагом в 1px. От 12px до 30px',
                en: 'Indicator font size in 1px. 12px to 30px',
            },
            defaultValue: fontSize[4][0],
            options: fontSize.map((item) => {
                const [value, ru, en] = item;
                return {
                    label: {
                        ru,
                        en,
                    },
                    value,
                };
            }),
        }),
        select({
            key: EViewOption.FontSizeDescription,
            label: {
                ru: 'Размер шрифта описания с шагом в 1px. От 12px до 30px',
                en: 'Description font size in 1px. 12px to 30px',
            },
            defaultValue: fontSize[4][0],
            options: fontSize.map((item) => {
                const [value, ru, en] = item;
                return {
                    label: {
                        ru,
                        en,
                    },
                    value,
                };
            }),
        }),
        select({
            key: EViewOption.FontSizeName,
            label: {
                ru: 'Размер шрифта наименования с шагом в 1px. От 12px до 30px',
                en: 'Name font size in 1px. 12px to 30px',
            },
            defaultValue: fontSize[4][0],
            options: fontSize.map((item) => {
                const [value, ru, en] = item;
                return {
                    label: {
                        ru,
                        en,
                    },
                    value,
                };
            }),
        }),
        input({
            key: EViewOption.Suffix,
            label: {
                ru: 'Суффикс для показателя',
                en: 'Indicator suffix',
            },
        }),
        checkbox({
            key: EViewOption.EnableColorIndicator,
            label: {
                ru: 'Цветовая дифференциация по шкале',
                en: 'Color differentiation by interval',
            },
            defaultValue: false
        }),
        ...((viewSettings[EViewOption.EnableColorIndicator])?[input({
            key: EViewOption.IndicatorStepCount,
            label: {
                ru: 'Количество цветов шкалы',
                en: 'Indicator suffix',
            },
            defaultValue:'2'
        }),]:[]),
        ...generate_steps(parseInt(viewSettings[EViewOption.IndicatorStepCount]),viewSettings[EViewOption.EnableColorIndicator])
        ,
    ];
};

function generate_steps(count:number,active:boolean):ViewSettingsItem[] {
    if(!active||count<2)return [];
    let ret:ViewSettingsItem[]=[];
    ret.push(        
        colorPicker({
            key: EViewOption.IndicatorStepColor+`_${1}`,
            label: {
                ru: `Цвет, если строго меньше значения ${1}`,
                en: `Color for ${1} step`,
            },
            defaultValue: `0`,
        }),
        input({
            key: EViewOption.IndicatorStep+`_${1}`,
            label: {
                ru: `Значение ${1} `,
                en: `${1} step value`,
            },
            defaultValue: `0`,
        }),
    )
    for(let i=2;i<count;i++){
        ret.push(        
            colorPicker({
                key: EViewOption.IndicatorStepColor+`_${i}`,
                label: {
                    ru: `Цвет, если строго меньше значения ${i}, но больше или равно значению ${i-1} `,
                    en: `Color for ${i} step`,
                },
                defaultValue: `0`,
            }),
            input({
                key: EViewOption.IndicatorStep+`_${i}`,
                label: {
                    ru: `Значение ${i}  `,
                    en: `${i} step value`,
                },
                defaultValue: `0`,
            }),
        )
    }
    ret.push(        
        colorPicker({
            key: EViewOption.IndicatorStepColor+`_${count}`,
            label: {
                ru: `Цвет, если больше или равно значению ${count-1}`,
                en: `Color for ${count - 1 } step`,
            },
        }))
    return ret;
}

export const validation: ViewSettingsValidation = {
    [EViewOption.DigitCapacity]: (value) => {
        const valueFloat = parseFloat(value);
        if (
            !/^\d+$/.test(value) ||
            valueFloat < 0 ||
            valueFloat > digitCapacityMax
        ) {
            return {
                ru: `укажите целое число от 0 до ${digitCapacityMax}`,
                en: `enter an integer from 0 to ${digitCapacityMax}`,
            };
        }
        return null;
    },
    [EViewOption.IndicatorStepCount]:(value)=>{
        const valueFloat = parseFloat(value);
        if (
            !/^\d+$/.test(value) ||
            valueFloat < 2 
        ) {
            return {
                ru: `укажите целое число больше 2`,
                en: `enter an integer from 0 `,
            };
        }
        return null;
    }
};
